@@ -21,10 +21,11 @@
 # Where do we get up-to-date seeds from?
 seeds = 'http://people.canonical.com/~ubuntu-archive/seeds/'
 seeds_bzr = 'http://bazaar.launchpad.net/~ubuntu-core-dev/ubuntu-seeds/'
-release = 'ubuntu.trusty'
+seeds_git = 'git://git.launchpad.net/~ubuntu-core-dev/ubuntu-seeds/+git/'
+release = 'ubuntu.wily'
 
 # If we need to download Packages.gz and/or Sources.gz, where do we get
 # them from?
 mirror = 'http://archive.ubuntu.com/ubuntu/'
-dist = 'trusty'
+dist = 'wily'
 arch = 'i386'
